package test;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.persistence.Query;

import org.junit.Test;

import code.TestDataBean;

public class OpenJpaTest {
	
	@Test
	public void test() throws Exception {
		try {
		    EntityManagerFactory emf =  Persistence.createEntityManagerFactory("testdb");
		    EntityManager em = emf.createEntityManager();
		    EntityTransaction tx = em.getTransaction();
		    
		    // clear table
		    {
			    tx.begin();
			    List<TestDataBean> tdbList = findAll(em);
			    for (TestDataBean currTdb : tdbList) {
			    	em.remove(currTdb);
			    }
			    tx.commit();
		    }
		    
		    TestDataBean newTestDataBean = null;
		    // create a test object
		    {
			    tx.begin();
			    newTestDataBean = new TestDataBean("testFirstName", "testLastName");
			    em.persist(newTestDataBean);
			    tx.commit();
			    System.out.println("Id: " + newTestDataBean.getId());
			    if (newTestDataBean.getId() == null || newTestDataBean.getId() == 0)
			    	throw new Exception();
		    }
		    
	    	// update the test object
		    {
		    	// modify @Entity
		    	newTestDataBean.setFirstName("MODIFIED-testFirstName");
		    	// modify @MappedSuperclass
		    	newTestDataBean.setLastEventTypeId(1001);
		    	tx.begin();
		    	newTestDataBean = em.merge(newTestDataBean);
		    	tx.commit();
		    }
		    
		    // re-load the test object and assert modified state
		    {
		    	Query query = em.createQuery("select tdb TestDataBean from TestDataBean tdb where tdb.id = ?1");
		    	query.setParameter(1, newTestDataBean.getId());
		    	List<TestDataBean> tdbList = query.getResultList();
		    	if (tdbList.size() != 1)
		    		throw new Exception();
		    	// Re-using reference
		    	newTestDataBean = tdbList.get(0);
		    	// Asserting state
		    	// Asserting modified @Entity state
		    	if (!newTestDataBean.getFirstName().equals("MODIFIED-testFirstName"))
		    		throw new Exception();
		    	// Asserting modified @MappedSuperclass state
		    	if (newTestDataBean.getLastEventTypeId() != 1001)
		    		throw new Exception();
		    }
		    
		    em.close();
		    emf.close();
		} catch (Throwable e) {
			e.printStackTrace();
			throw e;
		}
	}
	
	private List<TestDataBean> findAll(EntityManager em) throws Exception {
		Query query = em.createQuery("select tdb from TestDataBean tdb");
		List<TestDataBean> tdbList = query.getResultList();
		return tdbList;
	}
}
